﻿'Translation from C# to vb was done by JimmyTheGreat
'All comments and overall idea are still owned by Aureus

Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports XIP = XIPlugin
Imports System.Windows.Forms
Imports NLog
Imports System.Threading

'Namespace $safeprojectname$

Public Class XIPlugin
    Implements XIP.XIPluginInterface
    Implements XIP.XIMultiInstancePlugin

    ' Creates a NLog logger for this class.  Try to use the correct
    ' logging level when logging information (from lowest to highest):
    ' Trace: The lowest level, use for fine detail.  i.e. entering/exiting logging
    ' Debug: Information on the application flow.  Use for debugging info during
    ' development.  This is the lowest level that appears in the console by default.
    ' Info: Used for interesting runtime events.  Starting/stopped logging should go here
    ' Warn: Used for runtime conditions that are bad, but not necessarily a dealbreaker.  
    ' This is the lowest level that is logged to a file by default.
    ' Error: Pretty obvious, these are serious problems.
    ' 
    ' The inclusion of NLog is to encourage thorough logging.  Ideally, all a user should
    ' need to do in order to provide precise logs in the event of an error is to change to the 
    ' appropriate logging level and send in their text log.
    Private Shared logger As Logger = LogManager.GetCurrentClassLogger()

    Private settingsControl As SettingsControl

    ' This dictionary holds on to the ffxi instance list, as a dictionary 
    ' with the format:
    ' [processId => FFXIWindow]
    Private windows As Dictionary(Of Integer, XIP.FFXIWindow)

    ' The startPlugin call is used to setup any necessary data or 
    ' callbacks for the plugin.  This is where you would setup
    ' low level hardware callbacks for plugins with message support, or
    ' initialize any fface/windower text fields
    Public Sub startPlugin(ByVal mainApp As Form) Implements XIP.XIPluginInterface.startPlugin
        logger.Debug("Starting simple plugin")
    End Sub

    ' Plugin name.
    Public ReadOnly Property Name As String Implements XIP.XIPluginInterface.Name
        Get
            Return "$safeprojectname$"
        End Get
    End Property

    ' This method is called in different contexts depending on the
    ' plugin type.  For a multi instance plugin, this is called every time a 
    ' ffxi window is created or exits, along with on creation.
    ' For a single instance plugin, this is only called at creation.
    Public Sub setPluginContext(ByVal context As XIP.XIPluginContext) Implements XIP.XIPluginInterface.setPluginContext
        logger.Trace("Plugin Context updated")
        windows = context.windows
    End Sub

    ' This method is called when the plugin needs to terminate.  
    ' If the plugin thread is still active some time after calling this, 
    ' it will be force stopped.  Clean up any windower or fface
    ' text boxes here, take care of any timers, etc.
    Public Sub stopPlugin() Implements XIP.XIPluginInterface.stopPlugin
    End Sub

    ' This returns the keyboard command associated with this plugin.  Any 
    ' windower input matching this command will be sent to the processCommand 
    ' method in this plugin.  If no keyboard command is necessary for a plugin,
    ' return null from this method.
    Public ReadOnly Property Command As String Implements XIP.XIPluginInterface.Command
        Get
            Return "$safeprojectname$"
        End Get
    End Property

    ' This method is called when a matching keyboard command is found.  
    ' For example, the command for this plugin is "simple", typing
    ' "//simple test 1 2 3" into the chatline will call this method with a
    ' XICommandContext object with args of ["test", "1", "2", "3"]
    ' It is important to note that processCommand is called in a new
    ' thread, so be sure to use threadsafe access to data
    Public Sub processCommand(ByVal args As Object) Implements XIP.XIPluginInterface.processCommand
        Dim xic As xip.XICommandContext = CType(args, xip.XICommandContext)
        Dim commandArgs As List(Of String) = xic.args

        logger.Debug("CommandArgs from pid " & xic.pid & ": ")
        For Each Str As String In commandArgs
            logger.Debug(Str)
        Next
    End Sub

    ' Plugin description, this shows up in the plugin selection box
    Public ReadOnly Property Description As String Implements XIP.XIPluginInterface.Description
        Get
            Return "Simple Debugging Plugin in VB"
        End Get
    End Property

    ' This should return the display tab of the plugin, if one is needed.
    ' For plugins that do not require a display tab, return null.
    Public ReadOnly Property DisplayTab As TabPage Implements XIP.XIPluginInterface.DisplayTab
        Get
            Dim t As New TabPage(Name)
            t.Controls.Add(New TabControl())
            Return t
        End Get
    End Property

    ' This should return the settings tab of the plugin, if one is needed.
    ' For plugins that do not require a settings tab, return null.
    Public ReadOnly Property SettingsTab As TabPage Implements XIP.XIPluginInterface.SettingsTab
        Get
            If settingsControl Is Nothing Then
                settingsControl = New SettingsControl()
            ElseIf settingsControl.IsDisposed Then
                settingsControl = New SettingsControl()
            End If
            Dim t As New TabPage(Name)
            t.Controls.Add(settingsControl)
            Return t
        End Get
    End Property

    ' This method is called when settings should be saved.  It is expected that
    ' the plugin will read in settings from the SettingsTab, and save them as necessary.
    Public Sub saveSettings() Implements XIP.XIPluginInterface.saveSettings
        Dim settings As SettingsModel = SettingsModel.getInstance()

        settings.someCheckbox = settingsControl.someCheckbox.Checked

        logger.Debug("Saving settings")
        settings.SaveSettings()
    End Sub
End Class

'End Namespace